@@ -331,7 +331,7 @@ static void do_start(Service_T s) {
   
   if (s->start && (s->type!=TYPE_PROCESS || !Util_isProcessRunning(s))) {
     LogInfo("'%s' start: %s\n", s->name, s->start->arg[0]);
-    spawn(s, s->start, "Started");
+    spawn(s, s->start, NULL);
     /* We only wait for a process type, other service types does not have a pid file to watch */
     if (s->type == TYPE_PROCESS)
       wait_start(s);
@@ -361,7 +361,7 @@ static int do_stop(Service_T s) {
 
   if (s->stop && (s->type!=TYPE_PROCESS || Util_isProcessRunning(s))) {
     LogInfo("'%s' stop: %s\n", s->name, s->stop->arg[0]);
-    spawn(s, s->stop, "Stopped");
+    spawn(s, s->stop, NULL);
     if (s->type == TYPE_PROCESS) {
       /* Only wait for process service types */
       if (!wait_stop(s))
@@ -470,9 +470,9 @@ static void wait_start(Service_T s) {
   }
   
   if (!Util_isProcessRunning(s))
-    Event_post(s, EVENT_EXEC, STATE_FAILED, s->action_EXEC, "failed to start");
+    Event_post(s, Event_Exec, STATE_FAILED, s->action_EXEC, "failed to start");
   else
-    Event_post(s, EVENT_EXEC, STATE_SUCCEEDED, s->action_EXEC, "started");
+    Event_post(s, Event_Exec, STATE_SUCCEEDED, s->action_EXEC, "started");
 
   return;
 }
@@ -499,10 +499,10 @@ static int wait_stop(Service_T s) {
   }
 
   if (Util_isProcessRunning(s)) {
-    Event_post(s, EVENT_EXEC, STATE_FAILED, s->action_EXEC, "failed to stop");
+    Event_post(s, Event_Exec, STATE_FAILED, s->action_EXEC, "failed to stop");
     return FALSE;
   } else {
-    Event_post(s, EVENT_EXEC, STATE_SUCCEEDED, s->action_EXEC, "stopped");
+    Event_post(s, Event_Exec, STATE_SUCCEEDED, s->action_EXEC, "stopped");
   }
 
   return TRUE;
