@@ -141,6 +141,7 @@ void gc_mail_list(Mail_T *m) {
 
   FREE((*m)->to);
   FREE((*m)->from);
+  FREE((*m)->replyto);
   FREE((*m)->subject);
   FREE((*m)->message);
   FREE(*m);
@@ -282,14 +283,11 @@ static void _gc_service(Service_T *s) {
   if((*s)->eventlist)
     gc_event(&(*s)->eventlist);
 
-  FREE((*s)->token);
   FREE((*s)->name);
   FREE((*s)->path);
   
   (*s)->next= NULL;
 
-  assert(pthread_mutex_destroy(&(*s)->mutex) == 0);
-
   FREE(*s);
 
 }
@@ -461,12 +459,8 @@ static void _gcpql(Resource_T *q) {
 
 
 static void _gc_inf(Info_T *i) {
-
   ASSERT(i);
-
-  FREE((*i)->cs_sum);
   FREE(*i);
-
 }
 
 
@@ -536,15 +530,12 @@ static void _gcmatch(Match_T *s) {
 
 
 static void _gcchecksum(Checksum_T *s) {
-  
   ASSERT(s);
 
-  if((*s)->action)
+  if ((*s)->action)
     _gc_eventaction(&(*s)->action);
 
-  FREE((*s)->hash);
   FREE(*s);
-
 }
 
 
