@@ -128,6 +128,19 @@ pthread_t       heartbeatThread;               /**< M/Monit heartbeat thread */
 pthread_cond_t  heartbeatCond;                /**< Hearbeat wakeup condition */
 pthread_mutex_t heartbeatMutex;                          /**< Hearbeat mutex */
 
+char actionnames[][STRLEN]   = {"ignore", "alert", "restart", "stop", "exec", "unmonitor", "start", "monitor", ""};
+char modenames[][STRLEN]     = {"active", "passive", "manual"};
+char checksumnames[][STRLEN] = {"UNKNOWN", "MD5", "SHA1"};
+char operatornames[][STRLEN] = {"greater than", "less than", "equal to", "not equal to"};
+char operatorshortnames[][3] = {">", "<", "=", "!="};
+char monitornames[][STRLEN]  = {"not monitored", "monitored", "initializing"};
+char statusnames[][STRLEN]   = {"accessible", "accessible", "accessible", "running", "online with all services", "running", "accessible"};
+char servicetypes[][STRLEN]  = {"Filesystem", "Directory", "File", "Process", "Remote Host", "System", "Fifo"};
+char pathnames[][STRLEN]     = {"Path", "Path", "Path", "Pid file", "Path", "", "Path"};
+char icmpnames[19][STRLEN]   = {"Echo Reply", "", "", "Destination Unreachable", "Source Quench", "Redirect", "", "", "Echo Request", "", "", "Time Exceeded", "Parameter Problem", "Timestamp Request", "Timestamp Reply", "Information Request", "Information Reply", "Address Mask Request", "Address Mask Reply"};
+char sslnames[][STRLEN]      = {"auto", "v2", "v3", "tls"};
+
+
 
 
 /* ------------------------------------------------------------------ Public */
@@ -361,7 +374,7 @@ static void do_reinit() {
     monit_http(START_HTTP);
 
   /* send the monit startup notification */
-  Event_post(Run.system, EVENT_INSTANCE, STATE_CHANGED, Run.system->action_MONIT_RELOAD, "Monit reloaded");
+  Event_post(Run.system, Event_Instance, STATE_CHANGED, Run.system->action_MONIT_RELOAD, "Monit reloaded");
 
   if(Run.mmonits && ((status = pthread_create(&heartbeatThread, NULL, heartbeat, NULL)) != 0))
     LogError("%s: Failed to create the heartbeat thread -- %s\n", prog, strerror(status));
@@ -462,7 +475,7 @@ static void do_exit() {
     LogInfo("%s daemon with pid [%d] killed\n", prog, (int)getpid());
 
     /* send the monit stop notification */
-    Event_post(Run.system, EVENT_INSTANCE, STATE_CHANGED, Run.system->action_MONIT_STOP, "Monit stopped");
+    Event_post(Run.system, Event_Instance, STATE_CHANGED, Run.system->action_MONIT_STOP, "Monit stopped");
   }
   gc();
   exit(0);
@@ -522,7 +535,7 @@ static void do_default() {
       monit_http(START_HTTP);
     
     /* send the monit startup notification */
-    Event_post(Run.system, EVENT_INSTANCE, STATE_CHANGED, Run.system->action_MONIT_START, "Monit started");
+    Event_post(Run.system, Event_Instance, STATE_CHANGED, Run.system->action_MONIT_START, "Monit started");
 
     if(Run.mmonits && ((status = pthread_create(&heartbeatThread, NULL, heartbeat, NULL)) != 0))
       LogError("%s: Failed to create the heartbeat thread -- %s\n", prog, strerror(status));
@@ -695,7 +708,7 @@ static void help() {
  * Print version information
  */
 static void version() {
-  printf("This is monit version %s\n", VERSION);
+  printf("This is Monit version " VERSION "\n");
   printf("Copyright (C) 2000-2010 by Tildeslash Ltd.");
   printf(" All Rights Reserved.\n");
 }
@@ -709,17 +722,16 @@ static void *heartbeat(void *args) {
   sigset_t ns;
   struct timespec wait;
 
-  wait.tv_nsec = 0;
   set_signal_block(&ns, NULL);
   LogInfo("M/Monit heartbeat started\n");
   LOCK(heartbeatMutex)
   {
     while (! Run.stopped && ! Run.doreload) {
-      DEBUG("Sending M/Monit heartbeat\n");
       if ((status = handle_mmonit(NULL)) == HANDLER_SUCCEEDED)
         wait.tv_sec = time(NULL) + Run.polltime;
       else
         wait.tv_sec = time(NULL) + 1;
+      wait.tv_nsec = 0;
       pthread_cond_timedwait(&heartbeatCond, &heartbeatMutex, &wait);
     }
   }
