@@ -182,6 +182,8 @@
 #define RESOURCE_ID_CPUSYSTEM         13
 #define RESOURCE_ID_CPUWAIT           14
 #define RESOURCE_ID_TOTAL_CPU_PERCENT 15
+#define RESOURCE_ID_SWAP_PERCENT      16
+#define RESOURCE_ID_SWAP_KBYTE        17
 
 #define DIGEST_CLEARTEXT   1
 #define DIGEST_CRYPT       2
@@ -216,7 +218,6 @@
 #define HANDLER_MAX        HANDLER_MMONIT
 
 #define ICMP_ATTEMPT_COUNT      3         
-#define ICMP_MAX_COUNT          20
 
 
 /** ------------------------------------------------- Special purpose macros */
@@ -305,7 +306,6 @@ typedef struct myaction {
     Command_T exec;                    /**< Optional command to be executed  */
     unsigned  count;           /**< Event count needed to trigger the action */
     unsigned  cycles;    /**< Cycles during which count limit can be reached */
-    char description[PATH_MAX];               /**< Action description buffer */
 } *Action_T;
 
 
@@ -356,6 +356,7 @@ typedef struct mymmonit {
 typedef struct mymail {
   char *to;                         /**< Mail address for alert notification */
   char *from;                                     /**< The mail from address */
+  char *replyto;                              /**< Optional reply-to address */
   char *subject;                                       /**< The mail subject */
   char *message;                                       /**< The mail message */
   unsigned int events;  /*< Events for which this mail object should be sent */
@@ -391,26 +392,28 @@ typedef struct myauthentication {
 
 /** Defines process tree - data storage backend*/
 typedef struct myprocesstree {
-  int  pid;
-  int  ppid;
-  int  status_flag;
+  int           pid;
+  int           ppid;
+  int           status_flag;
+  time_t        starttime;
+  char         *cmdline;
   
-  int  visited;
-  int  children_num;
-  int  children_sum;
-  int  cpu_percent;
-  int  cpu_percent_sum;
+  int           visited;
+  int           children_num;
+  int           children_sum;
+  int           cpu_percent;
+  int           cpu_percent_sum;
   unsigned long mem_kbyte;
   unsigned long mem_kbyte_sum;
 
   /** For internal use */
-  double time;                                             /**< 1/10 seconds */
-  double time_prev;                                        /**< 1/10 seconds */
-  long  cputime;                                           /**< 1/10 seconds */
-  long  cputime_prev;                                      /**< 1/10 seconds */
+  double        time;                                      /**< 1/10 seconds */
+  double        time_prev;                                 /**< 1/10 seconds */
+  long          cputime;                                   /**< 1/10 seconds */
+  long          cputime_prev;                              /**< 1/10 seconds */
   
-  int parent;
-  int *children;
+  int           parent;
+  int          *children;
 } ProcessTree_T;
 
 
@@ -420,8 +423,11 @@ typedef struct mysysteminfo {
   int cpus;                                              /**< Number of CPUs */
   double loadavg[3];                                /**< Load average triple */
   unsigned long mem_kbyte_max;               /**< Maximal system real memory */
+  unsigned long swap_kbyte_max;                               /**< Swap size */
   unsigned long total_mem_kbyte; /**< Total real memory in use in the system */
+  unsigned long total_swap_kbyte;       /**< Total swap in use in the system */
   int    total_mem_percent;      /**< Total real memory in use in the system */
+  int    total_swap_percent;            /**< Total swap in use in the system */
   int    total_cpu_user_percent;   /**< Total CPU in use in user space (pct.)*/
   int    total_cpu_syst_percent; /**< Total CPU in use in kernel space (pct.)*/
   int    total_cpu_wait_percent;      /**< Total CPU in use in waiting (pct.)*/
@@ -590,7 +596,7 @@ typedef struct mysize {
 
 /** Defines checksum object */
 typedef struct mychecksum {
-  char *hash;                     /**< A checksum hash computed for the path */
+  MD_T  hash;                     /**< A checksum hash computed for the path */
   int   type;                       /**< The type of hash (e.g. md5 or sha1) */
   int   length;                                      /**< Length of the hash */
   int   test_changes;           /**< TRUE if we only should test for changes */
@@ -676,8 +682,8 @@ typedef struct myinfo {
   /* File specific */
   off_t st_size;                                                   /**< Size */
   off_t readpos;                            /**< Position for regex matching */
-  ino_t  st_ino_prev;                 /**< Previous inode for regex matching */
-  char  *cs_sum;                                               /**< Checksum */
+  ino_t st_ino_prev;                  /**< Previous inode for regex matching */
+  MD_T  cs_sum;                                                /**< Checksum */
 
   /* Process specific */
   int    _pid;                              /**< Process PID from last cycle */
@@ -749,12 +755,12 @@ typedef struct myservice {
   EventAction_T action_ACTION;           /**< Action requested by CLI or GUI */
 
   /** Runtime parameters */
-  unsigned long long error;                          /**< Error flags bitmap */
-  unsigned long long error_hint;   /**< Failed/Changed hint for error bitmap */
+  int                error;                          /**< Error flags bitmap */
+  int                error_hint;   /**< Failed/Changed hint for error bitmap */
   Info_T             inf;                          /**< Service check result */
   struct timeval     collected;                /**< When were data collected */
   int                doaction;          /**< Action scheduled by http thread */
-  char              *token;                                /**< Action token */
+  char               token[STRLEN];                        /**< Action token */
 
   /** Events */
   struct myevent {
@@ -847,15 +853,21 @@ struct myrun {
   MailServer_T mailservers;    /**< List of MTAs used for alert notification */
   Mmonit_T mmonits;        /**< Event notification and status receivers list */
   Auth_T credentials;    /** A list holding Basic Authentication information */
+  int dommonitcredentials;   /**< TRUE if M/Monit should receive credentials */
+  Auth_T mmonitcredentials;     /**< Pointer to selected credentials or NULL */
   Event_T eventlist;              /** A list holding partialy handled events */
                                       /** User selected standard mail format */
   struct myformat {
     char *from;                          /**< The standard mail from address */
+    char *replyto;                             /**< Optional reply-to header */
     char *subject;                            /**< The standard mail subject */
     char *message;                            /**< The standard mail message */
   } MailFormat;
                                           
   pthread_mutex_t mutex;    /**< Mutex used for service data synchronization */
+#ifdef OPENSSL_FIPS
+  int fipsEnabled;                /** TRUE if monit should use FIPS-140 mode */
+#endif
 };
 
 
@@ -872,6 +884,18 @@ extern int            ptreesize;
 extern ProcessTree_T *oldptree;  
 extern int            oldptreesize; 
 
+extern char actionnames[][STRLEN];
+extern char modenames[][STRLEN];
+extern char checksumnames[][STRLEN];
+extern char operatornames[][STRLEN];
+extern char operatorshortnames[][3];
+extern char monitornames[][STRLEN];
+extern char statusnames[][STRLEN];
+extern char servicetypes[][STRLEN];
+extern char pathnames[][STRLEN];
+extern char icmpnames[19][STRLEN];
+extern char sslnames[][STRLEN];
+
 /* ------------------------------------------------------- Public prototypes */
 
 #include "util.h"
@@ -885,7 +909,7 @@ int   control_service_string(const char *, const char *);
 int   control_service_daemon(const char *, const char *);
 void  setup_dependants();
 void  reset_depend();
-void  spawn(Service_T, Command_T, const char *);
+void  spawn(Service_T, Command_T, Event_T);
 int   status(char *);
 int   log_init();
 void  LogEmergency(const char *, ...);
@@ -942,7 +966,7 @@ int  sha_md5_stream (FILE *, void *, void *);
 void reset_procinfo(Service_T);
 int  check_service_status(Service_T);
 void printhash(char *);  
-char *status_xml(Event_T, short);
+char *status_xml(Event_T, short, int);
 int  handle_mmonit(Event_T);
 int  do_wakeupcall();
 
