@@ -37,6 +37,10 @@
 #include <stdlib.h>
 #endif
 
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#endif
+
 #ifdef HAVE_STDARG_H
 #include <stdarg.h>
 #endif
@@ -49,12 +53,6 @@
 #define _BSD_SOCKLEN_T_
 #endif
 
-#ifdef HAVE_SOL_IP
-#define ICMP_SIZE sizeof(struct icmphdr)
-#else
-#define ICMP_SIZE sizeof(struct icmp)
-#endif
-
 #ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
 #endif
@@ -123,6 +121,12 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#else
+#define offsetof(st, m) ((size_t) ( (char *)&((st *)(0))->m - (char *)0 ))
+#endif
+
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -148,6 +152,12 @@
  */
 
 
+/* ------------------------------------------------------------- Definitions */
+
+
+#define DATALEN 64
+
+
 /* -------------------------------------------------------------- Prototypes */
 
 
@@ -363,10 +373,10 @@ int create_unix_socket(const char *pathname, int timeout) {
 int create_server_socket(int port, int backlog, const char *bindAddr) {
   int s;
   int status;
-  int flag= 1;
+  int flag = 1;
   struct sockaddr_in myaddr;
 
-  if((s= socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+  if((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
     LogError("%s: Cannot create socket -- %s\n", prog, STRERROR);
     return -1;
   }
@@ -419,7 +429,8 @@ int create_server_socket(int port, int backlog, const char *bindAddr) {
   return s;
 
   error:
-  close(s);
+  if (close(s) < 0)
+    LogError("%s: Socket %d close failed -- %s\n", prog, s, STRERROR);
 
   return -1;
 
@@ -432,17 +443,18 @@ int create_server_socket(int port, int backlog, const char *bindAddr) {
  * @return TRUE if the close succeed otherwise FALSE
  */
 int close_socket(int socket) {
-
   int r;
 
   shutdown(socket, 2);
   
+  /* Try to close even if shutdown failed so we won't leak file descriptors */
   do {
-    r= close(socket);
+    r = close(socket);
   } while(r == -1 && errno == EINTR);
+  if (r == -1)
+    LogError("%s: Socket %d close failed -- %s\n", prog, socket, STRERROR);
   
   return r;
-
 }
 
 
@@ -487,22 +499,15 @@ int set_block(int socket) {
  * @return Return TRUE if the event occured, otherwise FALSE.
  */
 int can_read(int socket, int timeout) {
+  int r = 0;
+  struct pollfd fds[1];
 
-  int r= 0;
-  fd_set rset;
-  struct timeval tv;
-  
-  FD_ZERO(&rset);
-  FD_SET(socket, &rset);
-  tv.tv_sec= timeout;
-  tv.tv_usec= 0;
-  
+  fds[0].fd = socket;
+  fds[0].events = POLLIN;
   do {
-    r= select(socket+1, &rset, NULL, NULL, &tv);
-  } while(r == -1 && errno == EINTR);
-
+    r = poll(fds, 1, timeout * 1000);
+  } while (r == -1 && errno == EINTR);
   return (r > 0);
-
 }
 
 
@@ -514,22 +519,15 @@ int can_read(int socket, int timeout) {
  * @return Return TRUE if the event occured, otherwise FALSE.
  */
 int can_write(int socket, int timeout) {
+  int r = 0;
+  struct pollfd fds[1];
 
-  int r= 0;
-  fd_set wset;
-  struct timeval tv;
-
-  FD_ZERO(&wset);
-  FD_SET(socket, &wset);
-  tv.tv_sec= timeout;
-  tv.tv_usec= 0;
-
+  fds[0].fd = socket;
+  fds[0].events = POLLOUT;
   do {
-    r= select(socket+1, NULL, &wset, NULL, &tv);
-  } while(r == -1 && errno == EINTR);
-
+    r = poll(fds, 1, timeout * 1000);
+  } while (r == -1 && errno == EINTR);
   return (r > 0);
-  
 }
 
 
@@ -653,153 +651,136 @@ int udp_write(int socket, void *b, int len, int timeout) {
  * @return response time on succes, -1 on error
  */
 double icmp_echo(const char *hostname, int timeout, int count) {
-  struct sockaddr_in sin;
   struct sockaddr_in sout;
   struct sockaddr_in *sa;
   struct addrinfo hints;
   struct addrinfo *result;
-#ifdef HAVE_SOL_IP
-  struct iphdr *iphdrin;
-  struct icmphdr *icmphdrin= NULL;
-  struct icmphdr *icmphdrout[ICMP_MAX_COUNT];
-#else
   struct ip *iphdrin;
-  struct icmp *icmphdrin= NULL;
-  struct icmp *icmphdrout[ICMP_MAX_COUNT];
-#endif
-  fd_set rset;
-  int i;
-  int s;
-  int n= 0;
-  int sol_ip;
-  unsigned ttl= 255;
+  int len_out = offsetof(struct icmp, icmp_data) + DATALEN;
+  int len_in = sizeof(struct ip) + sizeof(struct icmp);
+  struct icmp *icmpin = NULL;
+  struct icmp *icmpout = NULL;
+  uint16_t id_out;
+  int r, i, s, n = 0;
+  struct timeval t_out;
+  struct timeval t_in;
   char buf[STRLEN];
-  struct timeval tv;
-  struct timeval t1[ICMP_MAX_COUNT];
-  struct timeval t2;
-  double response= -1.;
+  double response = -1.;
+#if ! defined NETBSD && ! defined AIX
+  int sol_ip;
+  unsigned ttl = 255;
+#endif
   
   ASSERT(hostname);   
-  ASSERT(count <= ICMP_MAX_COUNT);
+  ASSERT(len_out < sizeof(buf));
 
   memset(&hints, 0, sizeof(struct addrinfo));
   hints.ai_family = AF_INET;
-  if(getaddrinfo(hostname, NULL, &hints, &result) != 0)
-    return response;
-
-  if((s= socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0) {
-    freeaddrinfo(result);
+  if (getaddrinfo(hostname, NULL, &hints, &result) != 0) {
+    LogError("ICMP echo -- getaddrinfo failed: %s\n", STRERROR);
     return response;
   }
 
+  if ((s = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0) {
+    LogError("ICMP echo -- socket failed: %s\n", STRERROR);
+    goto error2;
+  }
+
+#if ! defined NETBSD && ! defined AIX
 #ifdef HAVE_SOL_IP
   sol_ip = SOL_IP;
 #else
   {
     struct protoent *pent;
-    pent = getprotobyname( "ip" );
-    sol_ip = ( pent != NULL ) ? pent->p_proto : 0;
+    pent = getprotobyname("ip");
+    sol_ip = pent ? pent->p_proto : 0;
+  }
+#endif
+  if (setsockopt(s, sol_ip, IP_TTL, (char *)&ttl, sizeof(ttl)) < 0) {
+    LogError("ICMP echo -- setsockopt failed: %s\n", STRERROR);
+    goto error1;
   }
 #endif
 
-  if(setsockopt(s, sol_ip, IP_TTL, (char *)&ttl, sizeof(ttl)) < 0) {
-    freeaddrinfo(result);
-    goto error2;
-  }
+  id_out = (getpid() + time(NULL)) & 0xFFFF;
+  icmpout = (struct icmp *)buf;
+  for (i = 0; i < count; i++) {
+    int j;
+    unsigned char *data = (unsigned char *)icmpout->icmp_data;
+
+    icmpout->icmp_code  = 0;
+    icmpout->icmp_type  = ICMP_ECHO;
+    icmpout->icmp_id    = id_out;
+    icmpout->icmp_seq   = i;
+    icmpout->icmp_cksum = 0;
+
+    /* Add originate timestamp to data section */
+    gettimeofday(&t_out, NULL);
+    memcpy(data, &t_out, sizeof(struct timeval));
+    data += sizeof(struct timeval);
+
+    /* Initialize rest of data section to numeric sequence */
+    for (j = 0; j < DATALEN - sizeof(struct timeval); j++)
+      data[j] = j;
+
+    icmpout->icmp_cksum = checksum_ip((unsigned char *)icmpout, len_out);
 
-  tv.tv_sec= timeout;
-  tv.tv_usec= 0;
-  
-  for(i=0; i<count; i++) {
-    NEW(icmphdrout[i]);
-#ifdef HAVE_SOL_IP
-    icmphdrout[i]->code= 0;
-    icmphdrout[i]->type= ICMP_ECHO;
-    icmphdrout[i]->un.echo.id= getpid() + (long)hostname + time(NULL);
-    icmphdrout[i]->un.echo.sequence= i;
-    icmphdrout[i]->checksum= checksum_ip((unsigned char *)icmphdrout[i], ICMP_SIZE);
-#else
-    icmphdrout[i]->icmp_code= 0;
-    icmphdrout[i]->icmp_type= ICMP_ECHO;
-    icmphdrout[i]->icmp_id= getpid() + (long)hostname + time(NULL);
-    icmphdrout[i]->icmp_seq= i;
-    icmphdrout[i]->icmp_cksum= checksum_ip((unsigned char *)icmphdrout[i], ICMP_SIZE);
-#endif
     sa = (struct sockaddr_in *)result->ai_addr;
     memcpy(&sout, sa, result->ai_addrlen);
-    sout.sin_family= AF_INET;
-    sout.sin_port= 0;
-
-    /* Get time of particular connection attempt beginning */
-    gettimeofday(&t1[i], NULL);
+    sout.sin_family = AF_INET;
+    sout.sin_port   = 0;
 
     do {
-      n= sendto(s, (char *)icmphdrout[i], ICMP_SIZE, 0,
-	      (struct sockaddr *)&sout, sizeof(struct sockaddr));
+      n = sendto(s, (char *)icmpout, len_out, 0, (struct sockaddr *)&sout, sizeof(struct sockaddr));
     } while(n == -1 && errno == EINTR);
-  }
-  freeaddrinfo(result);
-  
-  do {
-
-    socklen_t size;
-
-    FD_ZERO(&rset);
-    FD_SET(s, &rset);
-
-    do {
-      n= select(s+1, &rset, NULL, NULL, &tv);
-    } while(n == -1 && errno == EINTR);
-
-    if(n <= 0)
-      goto error1;
-
-    size= sizeof(struct sockaddr_in);
-
-    do {
-      n= recvfrom(s, buf, STRLEN, 0, (struct sockaddr *)&sin, &size);
-    } while(n == -1 && errno == EINTR);
-    
-    if(n < 0)
-	    goto error1;
-    
-    for(i=0; i<count; i++) {
-#ifdef HAVE_SOL_IP
-      iphdrin= (struct iphdr *)buf;
-      icmphdrin= (struct icmphdr *)(buf + iphdrin->ihl * 4);
-      if( (icmphdrin->un.echo.id == icmphdrout[i]->un.echo.id) &&
-          (icmphdrin->type == ICMP_ECHOREPLY) &&
-          (icmphdrin->un.echo.sequence == icmphdrout[i]->un.echo.sequence) ) {
-#else
-      iphdrin= (struct ip *)buf;
-      icmphdrin= (struct icmp *)(buf + iphdrin->ip_hl * 4);
-      if( (icmphdrin->icmp_id == icmphdrout[i]->icmp_id) &&
-          (icmphdrin->icmp_type == ICMP_ECHOREPLY) &&
-          (icmphdrin->icmp_seq == icmphdrout[i]->icmp_seq) ) {
-#endif
-
-        /* Get time of connection attempt finish */
-        gettimeofday(&t2, NULL);
-
-        /* Get the response time */
-        response= (double)(t2.tv_sec  - t1[i].tv_sec) +
-                  (double)(t2.tv_usec - t1[i].tv_usec)/1000000;
-
-        goto done;
-      }
+    if (n < 0) {
+      LogError("ICMP echo request %d/%d failed -- %s\n", i + 1, count, STRERROR);
+      continue;
     }
-  } while(TRUE);
+  
+    if (can_read(s, timeout)) {
+      socklen_t size = sizeof(struct sockaddr_in);
+
+      do {
+        n = recvfrom(s, buf, STRLEN, 0, (struct sockaddr *)&sout, &size);
+      } while(n == -1 && errno == EINTR);
+      if (n < 0) {
+        LogError("ICMP echo response %d/%d failed -- %s\n", i + 1, count, STRERROR);
+        continue;
+      } else if (n < len_in) {
+        LogError("ICMP echo response %d/%d failed -- received %d bytes, expected at least %d bytes\n", i + 1, count, n, len_in);
+        continue;
+      }
+
+      iphdrin = (struct ip *)buf;
+      icmpin  = (struct icmp *)(buf + iphdrin->ip_hl * 4);
+      if (icmpin->icmp_type == ICMP_ECHOREPLY) {
+        if (icmpin->icmp_id == id_out && icmpin->icmp_seq < (uint16_t)count) {
+          /* Get the response time */
+          gettimeofday(&t_in, NULL);
+          memcpy(&t_out, icmpin->icmp_data, sizeof(struct timeval));
+          response = (double)(t_in.tv_sec - t_out.tv_sec) + (double)(t_in.tv_usec - t_out.tv_usec) / 1000000;
+          DEBUG("ICMP echo response %d/%d succeeded -- received id=%d sequence=%d response_time=%fs\n", i + 1, count, icmpin->icmp_id, icmpin->icmp_seq, response);
+          break; // Wait for one response only
+        } else
+          LogError("ICMP echo response %d/%d error -- received id=%d (expected id=%d), received sequence=%d (expected sequence 0-%d)\n", i + 1, count, icmpin->icmp_id, id_out, icmpin->icmp_seq, count - 1);
+      } else
+        LogError("ICMP echo response %d/%d failed -- invalid ICMP response type: %x (%s)\n", i + 1, count, icmpin->icmp_type, icmpin->icmp_type < 19 ? icmpnames[icmpin->icmp_type] : "unknown");
+    } else
+      LogError("ICMP echo response %d/%d timed out -- no response within %d seconds\n", i + 1, count, timeout);
+  }
 
-  done:
   error1:
-  for(i=0; i<count; i++) {
-    FREE(icmphdrout[i]);
-  }    
+  do {
+    r = close(s);
+  } while(r == -1 && errno == EINTR);
+  if (r == -1)
+    LogError("%s: Socket %d close failed -- %s\n", prog, s, STRERROR);
+
   error2:
-  close_socket(s);
+  freeaddrinfo(result);
 
   return response;
- 
 }
 
 
@@ -809,46 +790,36 @@ double icmp_echo(const char *hostname, int timeout, int count) {
 /*
  * Do a non blocking connect, timeout if not connected within timeout seconds
  */
-static int do_connect(int s, const struct sockaddr *addr,
-		      socklen_t addrlen, int timeout) {
+static int do_connect(int s, const struct sockaddr *addr, socklen_t addrlen, int timeout) {
+  int error;
+  struct pollfd fds[1];
 
-  int n, error;
-  fd_set wset, rset;
-  struct timeval tv;
-
-  errno= 0;
-  error= 0;
-  
-  if(0 > (n= connect(s, addr, addrlen)))
-    if(errno != EINPROGRESS)
-      return -1;
-  
-  FD_ZERO(&rset);
-  FD_SET(s, &rset);
-  wset= rset;
-  tv.tv_sec= timeout;
-  tv.tv_usec= 0;
-  
-  if(select(s+1, NULL, &rset, &wset, &tv)==0) {
-    close(s);
-    errno= ETIMEDOUT;
+  switch (connect(s, addr, addrlen)) {
+    case 0:
+      return 0;
+    default:
+      if (errno != EINPROGRESS)
+        return -1;
+      break;
+  }
+  fds[0].fd = s;
+  fds[0].events = POLLIN|POLLOUT;
+  if (poll(fds, 1, timeout * 1000) == 0) {
+    errno = ETIMEDOUT;
     return -1;
   }
-
-  if(FD_ISSET(s, &rset) || FD_ISSET(s, &wset)) {
-    socklen_t len= sizeof(error);
-    if(getsockopt(s, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
-      return -1;
-  } else
+  if (fds[0].events & POLLIN || fds[0].events & POLLOUT) {
+    socklen_t len = sizeof(error);
+    if (getsockopt(s, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
+      return -1; // Solaris pending error
+  } else {
     return -1;
-
-  if(error) {
-    errno= error;
+  }
+  if (error) {
+    errno = error;
     return -1;
   }
-
   return 0;
-  
 }
 
 
