@@ -75,30 +75,6 @@
 
 /* ------------------------------------------------------------------ Public */
 
-char actionnames[][STRLEN]=   {"ignore", "alert", "restart", "stop", "exec",
-	                       "unmonitor", "start", "monitor", ""};
-char modenames[][STRLEN]=     {"active", "passive", "manual"};
-char checksumnames[][STRLEN]= {"UNKNOWN", "MD5", "SHA1"};
-char operatornames[][STRLEN]= {"greater than", "less than", "equal to",
-                               "not equal to"};
-char operatorshortnames[][3]= {">", "<", "=", "!="};
-char monitornames[][STRLEN]=  {"not monitored", "monitored", "initializing"};
-char statusnames[][STRLEN]=   {"accessible", "accessible", "accessible",
-                               "running", "online with all services",
-                               "running", "accessible"};
-char servicetypes[][STRLEN]=  {"Filesystem", "Directory", "File", "Process",
-                               "Remote Host", "System", "Fifo"};
-char pathnames[][STRLEN]=     {"Path", "Path", "Path", "Pid file", "Path", "",
-                               "Path"};
-char icmpnames[][STRLEN]=     {"Echo Reply", "", "", "Destination Unreachable",
-                               "Source Quench", "Redirect", "", "",
-                               "Echo Request", "", "", "Time Exceeded",
-                               "Parameter Problem", "Timestamp Request",
-                               "Timestamp Reply", "Information Request",
-                               "Information Reply", "Address Mask Request",
-                               "Address Mask Reply"};
-char sslnames[][STRLEN]=      {"auto", "v2", "v3", "tls"};
-
 
 /**
  * Initialize the proc information code
@@ -145,6 +121,7 @@ int update_process_data(Service_T s, ProcessTree_T *pt, int treesize, pid_t pid)
     /* save the previous ppid and set actual one */
     s->inf->_ppid             = s->inf->ppid;
     s->inf->ppid              = pt[leaf].ppid;
+    s->inf->uptime            = time(NULL) - pt[leaf].starttime;
     s->inf->children          = pt[leaf].children_sum;
     s->inf->mem_kbyte         = pt[leaf].mem_kbyte;
     s->inf->status_flag       = pt[leaf].status_flag;
@@ -162,6 +139,7 @@ int update_process_data(Service_T s, ProcessTree_T *pt, int treesize, pid_t pid)
 
   } else {
     s->inf->ppid              = 0;
+    s->inf->uptime            = 0;
     s->inf->children          = 0;
     s->inf->total_mem_kbyte   = 0;
     s->inf->total_mem_percent = 0;
@@ -191,12 +169,13 @@ int update_system_load(ProcessTree_T *pt, int treesize) {
       goto error1;
     }
 
-    /** Get real memory usage statistic */
+    /** Get memory usage statistic */
     if (! used_system_memory_sysdep(&systeminfo)) {
       LogError("'%s' statistic error -- memory usage gathering failed\n", Run.system->name);
       goto error2;
     }
-    systeminfo.total_mem_percent = (int)(1000 * (double)systeminfo.total_mem_kbyte / (double)systeminfo.mem_kbyte_max);
+    systeminfo.total_mem_percent  = (int)(1000 * (double)systeminfo.total_mem_kbyte / (double)systeminfo.mem_kbyte_max);
+    systeminfo.total_swap_percent = systeminfo.swap_kbyte_max ? (int)(1000 * (double)systeminfo.total_swap_kbyte / (double)systeminfo.swap_kbyte_max) : 0;
 
     /** Get CPU usage statistic */
     if (! used_system_cpu_sysdep(&systeminfo)) {
@@ -243,6 +222,9 @@ int initprocesstree(ProcessTree_T **pt_r, int *size_r, ProcessTree_T **oldpt_r,
     DEBUG("system statistic error -- cannot initialize the process tree => process resource monitoring disabled\n");
     Run.doprocess = FALSE;
     return -1;
+  } else if (Run.doprocess == FALSE) {
+    DEBUG("system statistic -- initialization of the process tree succeeded => process resource monitoring enabled\n");
+    Run.doprocess = TRUE;
   }
 
   pt    = *pt_r;
@@ -268,8 +250,10 @@ int initprocesstree(ProcessTree_T **pt_r, int *size_r, ProcessTree_T **oldpt_r,
       pt[i].cpu_percent  = 0;
     }
         
-    if (pt[i].pid == pt[i].ppid)
+    if (pt[i].pid == pt[i].ppid) {
+      pt[i].parent = i;
       continue;
+    }
 
     if ((pt[i].parent = findprocess(pt[i].ppid, pt, *size_r)) == -1) {
       /* Parent process wasn't found - on Linux this is normal: main process with PID 0 is not listed, similarly in FreeBSD jail.
@@ -304,6 +288,7 @@ int initprocesstree(ProcessTree_T **pt_r, int *size_r, ProcessTree_T **oldpt_r,
   }
 
   fillprocesstree(pt, root);
+  update_system_load(*pt_r, *size_r);
 
   return *size_r;
 }
@@ -344,10 +329,8 @@ void delprocesstree(ProcessTree_T ** reference, int size) {
       return;
 
   for (i = 0; i < size; i++) {
-    if (pt[i].children != NULL) {
-      FREE(pt[i].children);
-      pt[i].children = NULL;
-    }
+    FREE(pt[i].cmdline);
+    FREE(pt[i].children);
   }
 
   FREE(pt);
