@@ -81,17 +81,19 @@
 
 
 /**
- * Reads an process dependent entry or the proc files system
+ * Reads an process dependent entry or the proc filesystem
  * @param buf buffer to write to
  * @param buf_size size of buffer "buf"
  * @param name name of proc service
- * @param pid pid number of the process / or <0 if main directory
+ * @param pid number of the process / or <0 if main directory
+ * @param bytes_read number of bytes read to buffer
  * @return TRUE if succeeded otherwise FALSE.
  */
-int read_proc_file(char *buf, int buf_size, char * name, int pid) {
+int read_proc_file(char *buf, int buf_size, char *name, int pid, int *bytes_read) {
   int fd;
   char filename[STRLEN];
   int bytes;
+  int rv = FALSE;
 
   ASSERT(buf);
   ASSERT(name);
@@ -102,22 +104,26 @@ int read_proc_file(char *buf, int buf_size, char * name, int pid) {
     snprintf(filename, STRLEN, "/proc/%d/%s", pid, name);
     
   if ((fd = open(filename, O_RDONLY)) < 0) {
-    DEBUG("cannot open file %s -- %s\n", filename, STRERROR);
-    return FALSE;
+    DEBUG("%s: Cannot open proc file %s -- %s\n", prog, filename, STRERROR);
+    return rv;
   }
 
   if ((bytes = read(fd, buf, buf_size-1)) < 0) {
-    close(fd);
-    DEBUG("cannot read file %s -- %s\n", filename, STRERROR);
-    return FALSE;
+    DEBUG("%s: Cannot read proc file %s -- %s\n", prog, filename, STRERROR);
+    goto error;
   }
+  if (bytes_read)
+    *bytes_read = bytes;
        
   /* In case it is a string we have to 0 terminate it our self */
   buf[bytes]='\0';
+  rv = TRUE;
 
-  close(fd);
+error:
+  if (close(fd) < 0)
+    LogError("%s: Socket close failed -- %s\n", prog, STRERROR);
 
-  return TRUE;
+  return rv;
 }
 
 /**
@@ -177,7 +183,7 @@ void fillprocesstree(ProcessTree_T *pt, int index) {
   for (i = 0; i < pt[index].children_num; i++)
     fillprocesstree(pt, pt[index].children[i]);
 
-  if (pt[index].parent != -1) {
+  if (pt[index].parent != -1 && pt[index].parent != index) {
     parent_pt                   = &pt[pt[index].parent];
     parent_pt->children_sum    += pt[index].children_sum;
     parent_pt->mem_kbyte_sum   += pt[index].mem_kbyte_sum;
