@@ -84,6 +84,9 @@
  */
 
 
+#define ARGSSIZE 8192
+
+
 /* ----------------------------------------------------------------- Private */
 
 
@@ -94,6 +97,9 @@ static long cpu_user_old = 0;
 static long cpu_syst_old = 0;
 
 
+static char *skipString(char *p);
+
+
 /* ------------------------------------------------------------------ Public */
 
 
@@ -150,37 +156,79 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
   int                i;
   int                treesize;
   mach_port_t        mytask = mach_task_self();
-  struct kinfo_proc *pinfo;
   ProcessTree_T     *pt;
-  size_t             bufSize = 0;
-  int                mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_ALL, 0};
+  struct kinfo_proc *pinfo;
+  size_t             pinfo_size = 0;
+  char              *args;
+  size_t             args_size = 0;
+  size_t             size;
+  int                mib[4];
 
-  if (getuid()!=0) {
-    LogError("system statistic error -- permission denied\n");
+  mib[0] = CTL_KERN;
+  mib[1] = KERN_PROC;
+  mib[2] = KERN_PROC_ALL;
+  mib[3] = 0;
+  if (sysctl(mib, 4, NULL, &pinfo_size, NULL, 0) < 0) {
+    LogError("system statistic error -- sysctl failed: %s\n", STRERROR);
     return FALSE;
   }
-
-  if (sysctl(mib, 4, NULL, &bufSize, NULL, 0) < 0) {
-    LogError("system statistic error -- sysctl failed\n");
-    return FALSE;
-  }
-  pinfo = (struct kinfo_proc *)xcalloc(1, bufSize);
-
-  if (sysctl(mib, 4, pinfo, &bufSize, NULL, 0)) {
+  pinfo = (struct kinfo_proc *)xcalloc(1, pinfo_size);
+  if (sysctl(mib, 4, pinfo, &pinfo_size, NULL, 0)) {
     FREE(pinfo);
-    LogError("system statistic error -- sysctl failed\n");
+    LogError("system statistic error -- sysctl failed: %s\n", STRERROR);
     return FALSE;
   }
-
-  treesize = bufSize / sizeof(struct kinfo_proc);
-
+  treesize = pinfo_size / sizeof(struct kinfo_proc);
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
 
+  mib[0] = CTL_KERN;
+  mib[1] = KERN_ARGMAX;
+  size = sizeof(args_size);
+  if (sysctl(mib, 2, &args_size, &size, NULL, 0) == -1) {
+    FREE(pinfo);
+    FREE(pt);
+    LogError("system statistic error -- sysctl failed: %s\n", STRERROR);
+    return FALSE;
+  }
+  args = (char *)xcalloc(1, args_size);
+  size = args_size; // save for per-process sysctl loop
+
   for (i = 0; i < treesize; i++) {
     mach_port_t task;
 
-    pt[i].pid  = pinfo[i].kp_proc.p_pid;
-    pt[i].ppid = pinfo[i].kp_eproc.e_ppid;
+    pt[i].pid       = pinfo[i].kp_proc.p_pid;
+    pt[i].ppid      = pinfo[i].kp_eproc.e_ppid;
+    pt[i].starttime = pinfo[i].kp_proc.p_starttime.tv_sec;
+
+    args_size = size;
+    mib[0] = CTL_KERN;
+    mib[1] = KERN_PROCARGS2;
+    mib[2] = pt[i].pid;
+    if (sysctl(mib, 3, args, &args_size, NULL, 0) != -1) {
+      /* KERN_PROCARGS2 sysctl() returns following pseudo structure:
+       *        struct {
+       *                int argc
+       *                char execname[];
+       *                char argv[argc][];
+       *                char env[][];
+       *        }
+       * The strings are terminated with '\0' and may have variable '\0' padding
+       */
+      int       argc = *args;
+      char     *p = args + sizeof(int); // arguments beginning
+      Buffer_T  cmdline;
+
+      memset(&cmdline, 0, sizeof(Buffer_T));
+
+      p = skipString(p); // skip exec name
+      while (argc-- && p < args + args_size) {
+        Util_stringbuffer(&cmdline, argc ? "%s " : "%s", p);
+        p = skipString(p);
+      }
+      pt[i].cmdline = cmdline.buf;
+    }
+    if (! pt[i].cmdline || ! *pt[i].cmdline)
+      pt[i].cmdline = xstrdup(pinfo[i].kp_proc.p_comm);
 
     if (pinfo[i].kp_proc.p_stat == SZOMB)
       pt[i].status_flag |= PROCESS_ZOMBIE;
@@ -219,6 +267,7 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
       mach_port_deallocate(mytask, task); 	
     }
   }
+  FREE(args);
   FREE(pinfo);
 
   *reference = pt;
@@ -244,24 +293,39 @@ int getloadavg_sysdep (double *loadv, int nelem) {
  * @return: TRUE if successful, FALSE if failed (or not available)
  */
 int used_system_memory_sysdep(SystemInfo_T *si) {
+  int                    mib[2];
+  unsigned int           pw, pa, pi;
+  size_t                 len;
+  struct xsw_usage       swap;
   kern_return_t          kret;
   vm_statistics_data_t   page_info;
   mach_msg_type_number_t count;
 
+  /* Memory */
   count = HOST_VM_INFO_COUNT;
   kret  = host_statistics(mach_host_self(), HOST_VM_INFO, (host_info_t)&page_info, &count);
-  if (kret == KERN_SUCCESS) {
-    unsigned int pw, pa, pi;
-
-    pw = page_info.wire_count * pagesize_kbyte;
-    pa = page_info.active_count * pagesize_kbyte;
-    pi = page_info.inactive_count * pagesize_kbyte;
-
-    si->total_mem_kbyte = pw+pa+pi;
-
-    return TRUE;
+  if (kret != KERN_SUCCESS) {
+    DEBUG("system statistic error -- cannot get memory usage\n");
+    return FALSE;
   }
-  return FALSE;
+  pw = page_info.wire_count * pagesize_kbyte;
+  pa = page_info.active_count * pagesize_kbyte;
+  pi = page_info.inactive_count * pagesize_kbyte;
+  si->total_mem_kbyte = pw + pa + pi;
+
+  /* Swap */
+  mib[0] = CTL_VM;
+  mib[1] = VM_SWAPUSAGE;
+  len    = sizeof(struct xsw_usage);
+  if (sysctl(mib, 2, &swap, &len, NULL, 0) == -1) {
+    DEBUG("system statistic error -- cannot get swap usage: %s\n", STRERROR);
+    si->swap_kbyte_max = 0;
+    return FALSE;
+  }
+  si->swap_kbyte_max   = (unsigned long)(double)(swap.xsu_total) / 1024.;
+  si->total_swap_kbyte = (unsigned long)(double)(swap.xsu_used) / 1024.;
+
+  return TRUE;
 }
 
 
@@ -297,3 +361,11 @@ int used_system_cpu_sysdep(SystemInfo_T *si) {
   return FALSE;
 }
 
+
+static char *skipString(char *p) {
+  p += strlen(p);
+  while (! *p) // skip terminating '\0' and possible padding
+    p++;
+  return p;
+}
+
