@@ -50,6 +50,10 @@
 #include <sys/proc.h>
 #endif
 
+#ifdef HAVE_KVM_H
+#include <kvm.h>
+#endif
+
 #ifdef HAVE_UVM_UVM_H
 #include <uvm/uvm.h>
 #endif
@@ -144,10 +148,12 @@ int init_process_info_sysdep(void) {
 int initprocesstree_sysdep(ProcessTree_T **reference) {
   int                        i;
   int                        treesize;
+  char                       buf[_POSIX2_LINE_MAX];
   size_t                     size = sizeof(maxslp);
   int                        mib_proc2[6] = {CTL_KERN, KERN_PROC2, KERN_PROC_KTHREAD, 0, sizeof(struct kinfo_proc2), 0};
   static int                 mib_maxslp[] = {CTL_VM, VM_MAXSLP};
   ProcessTree_T             *pt;
+  kvm_t                     *kvm_handle;
   static struct kinfo_proc2 *pinfo;
 
   if (sysctl(mib_maxslp, 2, &maxslp, &size, NULL, 0) < 0) {
@@ -173,17 +179,37 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
 
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
 
+  if (! (kvm_handle = kvm_openfiles(NULL, NULL, NULL, KVM_NO_FILES, buf))) {
+    LogError("system statistic error -- kvm_openfiles failed: %s", buf);
+    return FALSE;
+  }
+
   for (i = 0; i < treesize; i++) {
+    int        j;
+    char     **args;
+    Buffer_T   cmdline;
+
     pt[i].pid         = pinfo[i].p_pid;
     pt[i].ppid        = pinfo[i].p_ppid;
+    pt[i].starttime   = pinfo[i].p_ustart_sec;
     pt[i].cputime     = (long)((pinfo[i].p_rtime_sec * 10) + (pinfo[i].p_rtime_usec / 100000));
     pt[i].cpu_percent = 0;
     pt[i].mem_kbyte   = (unsigned long)(pinfo[i].p_vm_rssize * pagesize_kbyte);
     if (pinfo[i].p_stat == SZOMB)
-      pt[i].status_flag |= PROCESS_ZOMBIE;
+      pt[i].status_flag |= PROCESS_ZOMBIE; //FIXME: save system service flag too (kernel threads)
     pt[i].time = get_float_time();
+
+    memset(&cmdline, 0, sizeof(Buffer_T));
+    if ((args = kvm_getargv2(kvm_handle, &pinfo[i], 0))) {
+      for (j = 0; args[j]; j++)
+        Util_stringbuffer(&cmdline, args[j + 1] ? "%s " : "%s", args[j]);
+      pt[i].cmdline = cmdline.buf;
+    }
+    if (! pt[i].cmdline || ! *pt[i].cmdline)
+      pt[i].cmdline = xstrdup(pinfo[i].p_comm);
   }
   FREE(pinfo);
+  kvm_close(kvm_handle);
 
   *reference = pt;
 
@@ -208,16 +234,32 @@ int getloadavg_sysdep (double *loadv, int nelem) {
  * @return: TRUE if successful, FALSE if failed (or not available)
  */
 int used_system_memory_sysdep(SystemInfo_T *si) {
-  int            mib[] = {CTL_VM, VM_METER};
-  size_t         len   = sizeof(struct vmtotal);
+  int            mib[2];
+  size_t         len;
   struct vmtotal vm;
+  struct uvmexp  vmexp;
 
+  /* Memory */
+  mib[0] = CTL_VM;
+  mib[1] = VM_METER;
+  len    = sizeof(struct vmtotal);
   if (sysctl(mib, 2, &vm, &len, NULL, 0) == -1) {
     LogError("system statistic error -- cannot get real memory usage: %s\n", STRERROR);
     return FALSE;
   }
-
   si->total_mem_kbyte = (unsigned long)(vm.t_arm * pagesize_kbyte);
+
+  /* Swap */
+  mib[1] = VM_UVMEXP;
+  len = sizeof(vmexp);
+  if (sysctl(mib, 2, &vmexp, &len, NULL, 0) == -1) {
+    LogError("system statistic error -- cannot get swap usage: %s\n", STRERROR);
+    si->swap_kbyte_max = 0;
+    return FALSE;
+  }
+  si->swap_kbyte_max   = (unsigned long)((double)vmexp.swpages   * ((double)vmexp.pagesize / 1024.));
+  si->total_swap_kbyte = (unsigned long)((double)vmexp.swpginuse * ((double)vmexp.pagesize / 1024.));
+
   return TRUE;
 }
 
