@@ -88,6 +88,10 @@
 #include <kstat.h>
 #endif
 
+#ifdef HAVE_SYS_SWAP_H
+#include <sys/swap.h>
+#endif
+
 #include <sys/sysinfo.h>
 
 #include "monitor.h"
@@ -106,12 +110,15 @@
 
 #define pagetok(size) ((size) << pageshift)
 
+static int    page_size;
 static int    pageshift=0;
 static long   old_cpu_user=0;
 static long   old_cpu_syst=0;
 static long   old_cpu_wait=0;
 static long   old_total=0;
 
+#define MAXSTRSIZE 80
+
 #ifndef LOG1024
 #define LOG1024         10
 #endif
@@ -132,6 +139,7 @@ int init_process_info_sysdep(void) {
   pageshift -= LOG1024;
 
   systeminfo.mem_kbyte_max = pagetok(sysconf(_SC_PHYS_PAGES));
+  page_size = getpagesize();
 
   return (TRUE);
 }
@@ -146,10 +154,11 @@ double timestruc_to_tseconds(timestruc_t t) {
  * the process tree (sysdep version... but should work for
  * all procfs based unices)
  * @param reference  reference of ProcessTree
- * @return treesize>=0 if succeeded otherwise <0.
+ * @return treesize>0 if succeeded otherwise =0.
  */
 int initprocesstree_sysdep(ProcessTree_T ** reference) {
   int            i;
+  int            rv;
   int            pid;
   int            treesize;
   char           buf[4096];
@@ -161,8 +170,10 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   ASSERT(reference);
 
   /* Find all processes in the /proc directory */
-  if (glob("/proc/[0-9]*", 0, NULL, &globbuf) != 0)
+  if ((rv = glob("/proc/[0-9]*", NULL, NULL, &globbuf)) != 0) {
+    LogError("system statistic error -- glob failed: %d (%s)\n", rv, STRERROR);
     return 0;
+  }
 
   treesize = globbuf.gl_pathc;
 
@@ -177,14 +188,15 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
     /* get the actual time */
     pt[i].time = get_float_time();
 
-    if (! read_proc_file(buf,4096, "psinfo", pt[i].pid)) {
+    if (! read_proc_file(buf, sizeof(buf), "psinfo", pt[i].pid, NULL)) {
       pt[i].cputime     = 0;
       pt[i].cpu_percent = 0;
       pt[i].mem_kbyte   = 0;
       continue;
     } 
 
-    pt[i].ppid = psinfo->pr_ppid;
+    pt[i].ppid      = psinfo->pr_ppid;
+    pt[i].starttime = psinfo->pr_start.tv_sec;
         
     /* If we don't have any light-weight processes (LWP) then we are definitely a zombie */
     if (psinfo->pr_nlwp == 0) {
@@ -196,8 +208,12 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
     } 
     
     pt[i].mem_kbyte = psinfo->pr_rssize;
-    
-    if (! read_proc_file(buf,4096, "status", pt[i].pid)) {
+
+    pt[i].cmdline  = xstrdup(psinfo->pr_psargs);
+    if (! pt[i].cmdline || ! *pt[i].cmdline)
+      pt[i].cmdline = xstrdup(psinfo->pr_fname);
+
+    if (! read_proc_file(buf, sizeof(buf), "status", pt[i].pid, NULL)) {
       pt[i].cputime     = 0;
       pt[i].cpu_percent = 0;
     } else {
@@ -232,10 +248,16 @@ int getloadavg_sysdep (double *loadv, int nelem) {
  * @return: TRUE if successful, FALSE if failed (or not available)
  */
 int used_system_memory_sysdep(SystemInfo_T *si) {
-  kstat_ctl_t   *kctl;  
-  kstat_named_t *knamed;
-  kstat_t       *kstat;
+  int                 i, n, num;
+  kstat_ctl_t        *kctl;  
+  kstat_named_t      *knamed;
+  kstat_t            *kstat;
+  swaptbl_t          *s;
+  char               *strtab;
+  unsigned long long  total = 0ULL;
+  unsigned long long  used  = 0ULL;
 
+  /* Memory */
   kctl  = kstat_open();
   kstat = kstat_lookup(kctl, "unix", 0, "system_pages");
   if (kstat_read(kctl, kstat, 0) == -1) {
@@ -243,13 +265,51 @@ int used_system_memory_sysdep(SystemInfo_T *si) {
     kstat_close(kctl);
     return FALSE;
   }
-  
   knamed = kstat_data_lookup(kstat, "freemem");
   if (knamed)
     si->total_mem_kbyte = systeminfo.mem_kbyte_max-pagetok(knamed->value.ul);
-  
   kstat_close(kctl);
 
+ /* Swap */
+again:
+ if ((num = swapctl(SC_GETNSWP, 0)) == -1) {
+    LogError("system statistic error -- swap usage gathering failed: %s\n", STRERROR);
+    return FALSE;
+  }
+  if (num == 0) {
+    DEBUG("system statistic -- no swap configured\n");
+    si->swap_kbyte_max = 0;
+    return TRUE;
+  }
+  s = (swaptbl_t *)xmalloc(num * sizeof(swapent_t) + sizeof(struct swaptable));
+  strtab = (char *)xmalloc((num + 1) * MAXSTRSIZE);
+  for (i = 0; i < (num + 1); i++)
+    s->swt_ent[i].ste_path = strtab + (i * MAXSTRSIZE);
+  s->swt_n = num + 1;
+  if ((n = swapctl(SC_LIST, s)) < 0) {
+    LogError("system statistic error -- swap usage gathering failed: %s\n", STRERROR);
+    si->swap_kbyte_max = 0;
+    FREE(s);
+    FREE(strtab);
+    return FALSE;
+  }
+  if (n > num) {
+    DEBUG("system statistic -- new swap added: deferring swap usage statistics to next cycle\n");
+    FREE(s);
+    FREE(strtab);
+    goto again;
+  }
+  for (i = 0; i < n; i++) {
+    if (!(s->swt_ent[i].ste_flags & ST_INDEL) && !(s->swt_ent[i].ste_flags & ST_DOINGDEL)) {
+      total += s->swt_ent[i].ste_pages;
+      used  += s->swt_ent[i].ste_pages - s->swt_ent[i].ste_free;
+    }
+  }
+  FREE(s);
+  FREE(strtab);
+  si->swap_kbyte_max   = (unsigned long)(double)(total * page_size) / 1024.;
+  si->total_swap_kbyte = (unsigned long)(double)(used  * page_size) / 1024.;
+
   return TRUE;
 }
 
