@@ -63,6 +63,7 @@
 #define P_LMTP           24
 #define P_GPS            25
 #define P_RADIUS         26
+#define P_MEMCACHE       27
 
 void  gc_protocols();
 
@@ -93,6 +94,7 @@ void* create_sip();
 void* create_lmtp();
 void* create_gps();
 void* create_radius();
+void* create_memcache();
 
 /* "Package" locale Protocol routines */
 int check_apache_status(Socket_T);
@@ -121,6 +123,7 @@ int check_sip(Socket_T);
 int check_lmtp(Socket_T);
 int check_gps(Socket_T);
 int check_radius(Socket_T);
+int check_memcache(Socket_T);
 
 
 #endif
