@@ -38,6 +38,9 @@
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+#ifdef OPENSSL_FIPS
+#include <openssl/fips.h>
+#endif
 #endif
 
 #define SSL_VERSION_AUTO       0
@@ -58,19 +61,19 @@ typedef struct myssl {
 
 
 typedef struct my_ssl_connection {
-  int            socket;
-  int            accepted;
-  SSL           *handler;
-  SSL_CTX       *ctx;
-  X509          *cert;
-  SSL_METHOD    *method;
-  BIO           *socket_bio;
-  const char    *cipher;
-  char          *cert_subject;
-  char          *cert_issuer;
-  unsigned char *cert_md5;
-  unsigned int   cert_md5_len;
-  char          *clientpemfile;
+  int               socket;
+  int               accepted;
+  SSL              *handler;
+  SSL_CTX          *ctx;
+  X509             *cert;
+  SSL_METHOD       *method;
+  BIO              *socket_bio;
+  const char       *cipher;
+  char             *cert_subject;
+  char             *cert_issuer;
+  unsigned char    *cert_md5;
+  unsigned int      cert_md5_len;
+  char             *clientpemfile;
 
   struct my_ssl_connection *prev;
   struct my_ssl_connection *next;
@@ -78,12 +81,12 @@ typedef struct my_ssl_connection {
 
 
 typedef struct my_ssl_server_connection {
-  int             server_socket;
-  SSL_METHOD     *method;
-  SSL_CTX        *ctx;
-  char           *pemfile;
-  char           *clientpemfile;
-  ssl_connection *ssl_conn_list;
+  int               server_socket;
+  SSL_METHOD       *method;
+  SSL_CTX          *ctx;
+  char             *pemfile;
+  char             *clientpemfile;
+  ssl_connection   *ssl_conn_list;
 } ssl_server_connection;
 
 
@@ -102,6 +105,9 @@ int                    recv_ssl_socket(ssl_connection *, void *, int, int);
 ssl_connection        *new_ssl_connection(char *, int);
 ssl_connection        *insert_accepted_ssl_socket(ssl_server_connection *);
 ssl_server_connection *init_ssl_server(char *, char *);
+#ifdef OPENSSL_FIPS
+void                   enable_fips_mode();
+#endif
 
 
 #else
