@@ -288,18 +288,25 @@ char *Util_getUptime(time_t delta, char *sep);
 
 
 /**
- * @return a checksum for the given file, or NULL if error.
+ * @return Store checksum for the given file in supplied buffer, return FALSE if failed, otherwise TRUE.
  */
-char *Util_getChecksum(char *file, int hashtype);
+int Util_getChecksum(char *file, int hashtype, char *buf, int bufsize);
 
 
 /**
- * Escape an uri string converting unsafe characters to a hex (%xx)
+ * Returns true if url contains url safe characters otherwise false
+ * @param url an url string to test
+ * @return true if url is url safe otherwise false
+ */
+int Util_isurlsafe(const char *url);
+
+/**
+ * Escape an url string converting unsafe characters to a hex (%xx)
  * representation.  The caller must free the returned string.
- * @param uri an uri string
+ * @param url an url string
  * @return the escaped string
  */
-char *Util_urlEncode(char *uri);
+char *Util_urlEncode(char *url);
 
 
 /**
@@ -311,6 +318,16 @@ char *Util_urlEncode(char *uri);
 char *Util_urlDecode(char *url);
 
 
+/**
+ * URL escape a service name so it can be safely transfeered over HTTP. In
+ * particular any '/' chars in name is encoded. The caller must free the 
+ * returned string.
+ * @param name a service name string to be URL encoded
+ * @return the escaped string
+ */
+char *Util_encodeServiceName(char *name);
+
+
 /**
  * @return a Basic Authentication Authorization string (RFC 2617),
  * with credentials from the Run object, NULL if credentials are not defined.
@@ -450,12 +467,24 @@ void Util_monitorUnset(Service_T s);
 int Util_getAction(const char *action);
 
 
-/**
- * Print event ratio needed to trigger the action to the buffer
- * @param action A action string
- * @return the action id
+/*
+ * Write full action description to given buffer
+ * @param action An action object
+ * @param buf Buffer
+ * @param bufsize Buffer size
+ * @return the buffer
  */
-char *Util_getEventratio(Action_T action, char *buf);
+char *Util_describeAction(Action_T action, char *buf, int bufsize);
+
+
+/**
+ * Print event ratio needed to trigger the action to given buffer
+ * @param action A action string
+ * @param buf Buffer
+ * @param bufsize Buffer size
+ * @return the buffer
+ */
+char *Util_getEventratio(Action_T action, char *buf, int bufsize);
 
 
 /**
