@@ -67,9 +67,10 @@
 /* -------------------------------------------------------------- Prototypes */
 
 
-static void document_head(Buffer_T *);
+static void document_head(Buffer_T *, int);
 static void document_foot(Buffer_T *);
-static void status_service(Service_T, Buffer_T *, short);
+static void status_service(Service_T, Buffer_T *, short, int);
+static void status_servicegroup(ServiceGroup_T, Buffer_T *, short);
 static void status_event(Event_T, Buffer_T *);
 
 
@@ -81,26 +82,32 @@ static void status_event(Event_T, Buffer_T *);
  * of monitored services and resources.
  * @param E An event object or NULL for general status
  * @param L Status information level
+ * @param V Format version
  * @return XML document or NULL in the case of error. The caller must free
 *  the memory.
  */
-char *status_xml(Event_T E, short L) {
+char *status_xml(Event_T E, short L, int V) {
   Buffer_T  B;
   Service_T S;
+  ServiceGroup_T SG;
 
   memset(&B, 0, sizeof(Buffer_T));
 
-  document_head(&B);
+  document_head(&B, V);
 
-  if (E) {
-    status_event(E, &B);
-  } else {
-    for (S = servicelist_conf; S; S = S->next_conf) {
-      LOCK(S->mutex)
-      status_service(S, &B, L);
-      END_LOCK;
-    }
+  if (V == 2)
+    Util_stringbuffer(&B, "<services>");
+  for (S = servicelist_conf; S; S = S->next_conf)
+    status_service(S, &B, L, V);
+  if (V == 2) {
+    Util_stringbuffer(&B, "</services>"
+                          "<servicegroups>");
+    for (SG = servicegrouplist; SG; SG = SG->next)
+      status_servicegroup(SG, &B, L);
+    Util_stringbuffer(&B, "</servicegroups>");
   }
+  if (E)
+    status_event(E, &B);
 
   document_foot(&B);
 
@@ -115,29 +122,40 @@ char *status_xml(Event_T E, short L) {
 /**
  * Prints a document header into the given buffer.
  * @param B Buffer object
+ * @param V Format version
  */
-static void document_head(Buffer_T *B) {
-
+static void document_head(Buffer_T *B, int V) {
   Util_stringbuffer(B,
-   "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"
-   "<monit>"
-   "<server>"
-   "<id>%s</id>"
-   "<incarnation>%lld</incarnation>"
-   "<version>%s</version>"
-   "<uptime>%ld</uptime>"
-   "<poll>%d</poll>"
-   "<startdelay>%d</startdelay>"
-   "<localhostname>%s</localhostname>"
-   "<controlfile>%s</controlfile>",
-   Run.id,
-   (long long)Run.incarnation,
-   VERSION,
-   (long)Util_getProcessUptime(Run.pidfile),
-   Run.polltime,
-   Run.startdelay,
-   Run.localhostname ? Run.localhostname : "",
-   Run.controlfile ? Run.controlfile : "");
+    "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
+  if (V == 2) {
+    Util_stringbuffer(B,
+      "<monit id=\"%s\" incarnation=\"%lld\" version=\"%s\">"
+      "<server>",
+      Run.id,
+      (long long)Run.incarnation,
+      VERSION);
+  } else {
+    Util_stringbuffer(B,
+      "<monit>"
+      "<server>"
+      "<id>%s</id>"
+      "<incarnation>%lld</incarnation>"
+      "<version>%s</version>",
+      Run.id,
+      (long long)Run.incarnation,
+      VERSION);
+  }
+  Util_stringbuffer(B,
+    "<uptime>%ld</uptime>"
+    "<poll>%d</poll>"
+    "<startdelay>%d</startdelay>"
+    "<localhostname>%s</localhostname>"
+    "<controlfile>%s</controlfile>",
+    (long)Util_getProcessUptime(Run.pidfile),
+    Run.polltime,
+    Run.startdelay,
+    Run.localhostname ? Run.localhostname : "",
+    Run.controlfile ? Run.controlfile : "");
 
   if(Run.dohttpd) {
     Util_stringbuffer(B,
@@ -149,6 +167,15 @@ static void document_head(Buffer_T *B) {
       Run.bind_addr?Run.bind_addr:"",
       Run.httpdport,
       Run.httpdssl);
+
+    if (Run.mmonitcredentials)
+        Util_stringbuffer(B,
+          "<credentials>"
+          "<username>%s</username>"
+          "<password>%s</password>"
+          "</credentials>",
+          Run.mmonitcredentials->uname,
+          Run.mmonitcredentials->passwd);
   }
  
   Util_stringbuffer(B,
@@ -160,13 +187,15 @@ static void document_head(Buffer_T *B) {
      "<machine>%s</machine>"
      "<cpu>%d</cpu>"
      "<memory>%lu</memory>"
+     "<swap>%lu</swap>"
      "</platform>",
      systeminfo.uname.sysname,
      systeminfo.uname.release,
      systeminfo.uname.version,
      systeminfo.uname.machine,
      systeminfo.cpus,
-     systeminfo.mem_kbyte_max);
+     systeminfo.mem_kbyte_max,
+     systeminfo.swap_kbyte_max);
 
 }
 
@@ -176,9 +205,7 @@ static void document_head(Buffer_T *B) {
  * @param B Buffer object
  */
 static void document_foot(Buffer_T *B) {
-
   Util_stringbuffer(B, "</monit>");
-
 }
 
 
@@ -187,38 +214,38 @@ static void document_foot(Buffer_T *B) {
  * @param S Service object
  * @param B Buffer object
  * @param L Status information level
+ * @param V Format version
  */
-static void status_service(Service_T S, Buffer_T *B, short L) {
-  ServiceGroup_T sg;
-  ServiceGroupMember_T sgm;
+static void status_service(Service_T S, Buffer_T *B, short L, int V) {
   Event_T E = S->eventlist;
 
+  if (V == 2)
+    Util_stringbuffer(B,
+      "<service name=\"%s\">"
+      "<type>%d</type>",
+      S->name ? S->name : "",
+      S->type);
+  else
+    Util_stringbuffer(B,
+      "<service type=\"%d\">"
+      "<name>%s</name>",
+      S->type,
+      S->name ? S->name : "");
   Util_stringbuffer(B,
-	    "<service type=\"%d\">"
-	    "<collected_sec>%ld</collected_sec>"
-	    "<collected_usec>%ld</collected_usec>"
-	    "<name>%s</name>"
-	    "<status>%llu</status>"
-	    "<status_hint>%llu</status_hint>"
-	    "<monitor>%d</monitor>"
-	    "<monitormode>%d</monitormode>"
-	    "<pendingaction>%d</pendingaction>",
-	    S->type,
-	    S->collected.tv_sec,
-	    S->collected.tv_usec,
-	    S->name ? S->name : "",
-	    S->error,
-	    S->error_hint,
-	    S->monitor,
-	    S->mode,
-	    S->doaction);
-
-  Util_stringbuffer(B, "<groups>");
-  for (sg = servicegrouplist; sg; sg = sg->next)
-    for (sgm = sg->members; sgm; sgm = sgm->next)
-      if (S->name && ! strcasecmp(sgm->name, S->name))
-        Util_stringbuffer(B, "<name>%s</name>", sg->name);
-  Util_stringbuffer(B, "</groups>");
+    "<collected_sec>%ld</collected_sec>"
+    "<collected_usec>%ld</collected_usec>"
+    "<status>%d</status>"
+    "<status_hint>%d</status_hint>"
+    "<monitor>%d</monitor>"
+    "<monitormode>%d</monitormode>"
+    "<pendingaction>%d</pendingaction>",
+    S->collected.tv_sec,
+    S->collected.tv_usec,
+    S->error,
+    S->error_hint,
+    S->monitor,
+    S->mode,
+    S->doaction);
 
   /* if the service is in error state, display first active error message to provide more details */
   while (E) {
@@ -270,8 +297,8 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
   		"</block>",
   		S->inf->flags,
   		S->inf->space_percent/10.,
-  		(float)S->inf->space_total / (float)1048576 * (float)S->inf->f_bsize,
-                (float)S->inf->f_blocks / (float)1048576 * (float)S->inf->f_bsize);
+  		S->inf->f_bsize > 0 ? (float)S->inf->space_total / (float)1048576 * (float)S->inf->f_bsize : 0,
+                S->inf->f_bsize > 0 ? (float)S->inf->f_blocks / (float)1048576 * (float)S->inf->f_bsize : 0);
         if(S->inf->f_files > 0) {
           Util_stringbuffer(B,
   		  "<inode>"
@@ -378,17 +405,23 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
 					"<percent>%.1f</percent>"
 					"<kilobyte>%ld</kilobyte>"
 					"</memory>"
+                                        "<swap>"
+                                        "<percent>%.1f</percent>"
+                                        "<kilobyte>%ld</kilobyte>"
+                                        "</swap>"
 					"</system>",
 					systeminfo.loadavg[0],
 					systeminfo.loadavg[1],
 					systeminfo.loadavg[2],
-					systeminfo.total_cpu_user_percent/10.,
-					systeminfo.total_cpu_syst_percent/10.,
+					systeminfo.total_cpu_user_percent > 0 ? systeminfo.total_cpu_user_percent/10. : 0,
+					systeminfo.total_cpu_syst_percent > 0 ? systeminfo.total_cpu_syst_percent/10. : 0,
 					#ifdef HAVE_CPU_WAIT
-					systeminfo.total_cpu_wait_percent/10.,
+					systeminfo.total_cpu_wait_percent > 0 ? systeminfo.total_cpu_wait_percent/10. : 0,
 					#endif
 					systeminfo.total_mem_percent/10.,
-					systeminfo.total_mem_kbyte);
+					systeminfo.total_mem_kbyte,
+                                        systeminfo.total_swap_percent/10.,
+                                        systeminfo.total_swap_kbyte);
       }
     }
   }
@@ -396,6 +429,22 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
 }
 
 
+/**
+ * Prints a servicegroups into the given buffer.
+ * @param SG ServiceGroup object
+ * @param B Buffer object
+ * @param L Status information level
+ */
+static void status_servicegroup(ServiceGroup_T SG, Buffer_T *B, short L) {
+  ServiceGroupMember_T SGM;
+
+  Util_stringbuffer(B, "<servicegroup name=\"%s\">", SG->name);
+  for (SGM = SG->members; SGM; SGM = SGM->next)
+    Util_stringbuffer(B, "<service>%s</service>", SGM->name);
+  Util_stringbuffer(B, "</servicegroup>");
+}
+
+
 /**
  * Prints a event description into the given buffer.
  * @param E Event object
@@ -421,13 +470,13 @@ static void status_event(Event_T E, Buffer_T *B) {
     "<message><![CDATA[%s]]></message>",
     tv->tv_sec,
     tv->tv_usec,
-    Event_get_id(E) == EVENT_INSTANCE ? "Monit" : Event_get_source_name(E),
+    Event_get_id(E) == Event_Instance ? "Monit" : Event_get_source_name(E),
     Event_get_source_type(E),
     Event_get_id(E),
     Event_get_state(E),
     Event_get_action(E),
     Event_get_message(E));
-  if ((s = Event_get_source(E)) && s->token)
+  if ((s = Event_get_source(E)) && *s->token)
     Util_stringbuffer(B, "<token>%s</token>", s->token);
   Util_stringbuffer(B,
     "</event>");
